#include <algorithm>
#include <vector>
#include "fiddle-simp.cc"
#include "c4spd.hh"

const double calc_c4spd(signed short* data, unsigned N)
{
    const double samprate = 11025;
    
    std::vector<double> pp(N);
    for(unsigned a=0; a<N; ++a)
        pp[a] = data[a] / 32768.0;

    const unsigned points   = std::max(MINPOINTS, std::min(MAXPOINTS,
                                  (1 << ((unsigned )(log2(N))))
                              ));
    const unsigned npitches = 1;
    
    const unsigned n_peaks_to_consider   = 40;
    const unsigned n_peaks_to_output_raw = 1;
    
    //fprintf(stderr, "points=%u\n", points);
    
    sigfiddle_class c(points, npitches, n_peaks_to_consider, n_peaks_to_output_raw);
    c.begin_dsp(samprate);
    const unsigned hop = points/2;
    for(unsigned n=0; n<N; n+=hop)
    {
        unsigned remain = N-n; if(remain > hop) remain = hop;
        c.put_dsp(&pp[n], remain);
        //c.dump();
    }
    
    //double noteout = c.x_noteout;
    double noteout = c.h_outlet[0].pitch;
    
    if(noteout == 0.0) return 0.0;

    while(noteout < 25) noteout += 12; // bring to sane range
    
    double freq = fiddle_mtof(noteout);
    double ratio = C4freq / freq;
    return samprate * ratio;
}
